
// Operations on level-0 ranges.

#include <stdlib.h>
#include <stdio.h>
#include <memory.h>
#include "globals.h"
#include "Procs.h"
#include "Range.h"


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//                                                                     //
// class `DimRangeRep': process dimension range.                       //
//                                                                     //
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

void DimRangeRep :: location(Location* loc, const int glb) const {
  loc->crd = glb ;
  loc->dim = dimension ;

  loc->sub = 0;
}

void DimRangeRep :: block(Block* blk, const int ker_glb) const {

  blk->sub_stp = 0 ;
  blk->glb_stp = 1 ;  // Feeds into subrange constructors: mustn't be zero.

  blk->count   = 1 ;

  blk->sub_bas = 0 ;
  blk->glb_bas = ker_glb ;
}

void DimRangeRep :: block2(Block* blk, const int ker_glb,
                           const int, const int base) const {
  blk->sub_stp = 0 ;
  blk->glb_stp = 1 ;  // Feeds into subrange constructors: mustn't be zero.

  blk->count   = 1 ;

  blk->sub_bas = 0 ;
  blk->glb_bas = ker_glb - base ;
}

void DimRangeRep :: block3(Block* blk, const int ker_glb,
                           const int, const int base,
                           const int stride) const {
  blk->sub_stp = 0 ;
  blk->glb_stp = 1 ;  // Feeds into subrange constructors: mustn't be zero.

  blk->count   = 1 ;

  blk->sub_bas = 0 ;
  blk->glb_bas = (ker_glb - base) / stride ;
}

void DimRangeRep :: crds(int* ker_count, int* ker_bas, int* ker_str) const {

  *ker_count = extent ;
  *ker_bas   = 0 ;
  *ker_str   = 1 ;
}

void DimRangeRep :: crds2(int* ker_count, int* ker_bas, int* ker_str,
                          const int count, const int base) const {
  *ker_count = count ;
  *ker_bas   = base ;
  *ker_str   = 1 ;
}

void DimRangeRep :: crds3(int* ker_count, int* ker_bas, int* ker_str,
                          const int count, const int base,
                          const int stride) const {
  *ker_count = count ;
  *ker_bas   = base ;
  *ker_str   = stride ;
}

int DimRangeRep :: volume() const {
  return 1 ;
}

int DimRangeRep :: offset(const Location*) const {
  return 0 ;
}

//int DimRangeRep :: disp(const int) const {
//  AdlibError("DimRangeRep :: disp: member undefined on this range") ;
//  return 0 ;
//}
//
//int DimRangeRep :: step(const int) const {
//  return 0 ;
//}

int DimRangeRep :: isAligned(RangeRep* x) const {
  return x == (RangeRep*) this ||
        (x->format() == DIST_PRIMITIVE && x->dim() == dimension &&
         x->size() == extent && x->str() == 1) ;
}

void DimRangeRep :: show() {
  AdlibPrintf("DimRangeRep :: show: size = %d\n", extent) ; 
}

#ifdef USE_POOL
MEMORY_POOL_DEFN(DimRangeRep)
#endif // USE_POOL


// -------------------------------------------------------------
// Things to do
//
//     Currently support ``collapsed dimensions'' through `dim == 0',
//         but don't support subranges of collapsed dimensions.
//         Not a problem?

